// -------------------------------------------------------------------------------------
//
// FishFa30.CPP                                                   Ulrich Mller 13.08.04
//                                                                PCS1700       VC++ 6.0
// CPP-Files fr die Klassen CFishFace ... 
//
// -------------------------------------------------------------------------------------

#include <windows.h>
#include "FishFa30.h"

// ---------- CFishFaceException Constructor -------------------------------

CFishFaceException::CFishFaceException(int nr, char* text) {
	aNr   = nr;
	aText = text;
}
// ---------- CFishFace Constructor ----------------------------------------

CFishFace::CFishFace() {
	aLPTAnalog    = 0;
	aLPTDelay     = 0;
	aSlave        = 0;
	aPollInterval = 0;
	aAnalogScan   = 0;
}
CFishFace::CFishFace(bool AnalogScan, bool Slave, int PollInterval) {
	aLPTAnalog    = 0;
	aLPTDelay     = 0;
	aSlave        = Slave ? 1 : 0;
	aPollInterval = PollInterval;
	aAnalogScan   = AnalogScan ? 1 : 0;
}
CFishFace::CFishFace(bool AnalogScan, bool Slave, int PollInterval,
										 int LPTAnalog, int LPTDelay) {
	aLPTAnalog    = LPTAnalog;
	aLPTDelay     = LPTDelay;
	aSlave        = Slave ? 1 : 0;
	aPollInterval = PollInterval;
	aAnalogScan   = AnalogScan ? 1 : 0;
}
CFishFace::~CFishFace() {}

// ---------- CFishFace public Methoden ------------------------------------

void CFishFace::ClearMotors() {
	if(!aOpen) throw CFishFaceException(2, "KeinOpen.ClearMotors");
	if(csSetMotors(ft, 0) == ftiError)
		throw CFishFaceException(3, "InterfaceProblem.ClearMotors");
}

void CFishFace::CloseInterface() {
	csCloseInterface(ft);
	aPortNr = -1;
	aOpen   = false;
}

bool CFishFace::Finish() {
	return Finish(0);
}

bool CFishFace::Finish(int InputNr) {
	if(!aOpen) throw CFishFaceException(4, "KeinOpen.Finish");
	if(aNotHalt || GetAsyncKeyState(VK_ESCAPE) != 0) return true;
	if(Inputs() == ftiError) throw CFishFaceException(5, "InterfaceProblem.Finish");
	if((InputNr > 0) && GetInput(InputNr)) return true;
	return false;
}

int CFishFace::GetAnalog(int AnalogNr) {
	if(!aOpen) throw CFishFaceException(5, "KeinOpen.GetAnalog");
	int res = csGetAnalog(ft, AnalogNr);
	if(res == ftiError) throw CFishFaceException(6, "InterfaceProblem.GetAnalog");
	return res;
}

int CFishFace::GetAnalogDirect(int AnalogNr) {
	if(!aOpen) throw CFishFaceException(7, "KeinOpen.GetAnalogDirect");
	int res = csGetAnalogDirect(ft, AnalogNr);
	if(res == ftiError) 
		throw CFishFaceException(8, "InterfaceProblem.GetAnalogDirect");
	return res;
}

bool CFishFace::GetInput(int InputNr) {
	if(!aOpen) throw CFishFaceException(9, "KeinOpen.GetInput");
	if(csGetInputs(ft) == ftiError)
		throw CFishFaceException(10, "InterfaceProblem.GetInput");
	return (csGetInput(ft, InputNr) == 0) ? false : true;
}

int CFishFace::GetInputs() {
	if(!aOpen) throw CFishFaceException(11, "KeinOpen.GetInputs");
	int res = csGetInputs(ft);
	if(res == ftiError) throw CFishFaceException(12, "InterfaceProblem.GetInputs");
	return res;
}

void CFishFace::OpenInterface(int PortNr) {
	if(PortNr < 0 || PortNr > 8) PortNr = 0;
	ft = csOpenInterfaceEx(PortNr, aAnalogScan, aSlave, aPollInterval,
		aLPTAnalog, aLPTDelay);
	if(ft == ftiError)
		throw CFishFaceException(13, "InterfaceProblem.OpenInterface");
	aPortNr = PortNr;
	csSetMotors(ft, 0);
	aNotHalt      = false;
	aOpen         = true;
	aAnalogScan   = csGetAnalogScan(ft);
	aLPTAnalog    = csGetLPTAnalog(ft);
	aLPTDelay     = csGetLPTDelay(ft);
	aSlave        = csGetSlave(ft);
	aPollInterval = csGetPollInterval(ft);
}

void CFishFace::Pause(unsigned int mSek){
	if(!aOpen) throw CFishFaceException(14, "KeinOpen.Pause");
	DWORD sZeit = GetTickCount();
	do {
		if(GetInputs() == ftiError)
			throw CFishFaceException(15, "InterfaceProblem.Pause");
		if(aNotHalt || GetAsyncKeyState(VK_ESCAPE) != 0) return;
		Sleep(aPollInterval);
	} while((GetTickCount() - sZeit) < mSek);
}

void CFishFace::SetLamp(int LampNr, int OnOff) {
	if(!aOpen) throw CFishFaceException(16, "KeinOpen.SetLamp");
	if(csSetLamp(ft, LampNr, OnOff) == ftiError)
		throw CFishFaceException(17, "InterfaceProblem.SetLamp");
}

void CFishFace::SetMotor(int MotorNr, int Direction) {
	if(!aOpen) throw CFishFaceException(17, "KeinOpen.SetMotor");
	if(csSetMotor(ft, MotorNr, Direction) == ftiError)
		throw CFishFaceException(18, "Interface.Problem");
}

void CFishFace::SetMotor(int MotorNr, int Direction, int Speed) {
	if(!aOpen) throw CFishFaceException(19, "KeinOpen.SetMotor");
	if(csSetMotorEx(ft, MotorNr, Direction, Speed) == ftiError)
		throw CFishFaceException(20, "InterfaceProblem.SetMotor");
}

void CFishFace::SetMotor(int MotorNr, int Direction, int Speed, int Counter) {
	if(!aOpen) throw CFishFaceException(21, "KeinOpen.SetMotor");
	if(csRobMotor(ft, MotorNr, Direction, Speed, Counter) == ftiError)
		throw CFishFaceException(22, "InterfaceProblem.SetMotor");
}

void CFishFace::SetMotors(int MotorStatus) {
	if(!aOpen) throw CFishFaceException(23, "KeinOpen.SetMotors");
	if(csSetMotors(ft, MotorStatus) == ftiError) 
		throw CFishFaceException(24, "InterfaceProblem.SetMotors");
}

void CFishFace::SetMotors(int MotorStatus, int SpeedStatus) {
	if(!aOpen) throw CFishFaceException(25, "KeinOpen.SetMotors");
	if(csSetMotorsEx(ft, MotorStatus, SpeedStatus) == ftiError) 
		throw CFishFaceException(26, "InterfaceProblem.SetMotors");
}

void CFishFace::SetMotors(int MotorStatus, int SpeedStatus, int ModeStatus) {
	if(!aOpen) throw CFishFaceException(27, "KeinOpen.SetMotors");
	if(csRobMotors(ft, MotorStatus, SpeedStatus, ModeStatus) == ftiError) 
		throw CFishFaceException(28, "InterfaceProblem.SetMotors");
}

void CFishFace::WaitForChange(int InputNr, int NrOfChanges, int TermInputNr) {
	if(!aOpen) throw CFishFaceException(29, "KeinOpen.WaitForChange");
	csSetCounter(ft, InputNr, 0);
	while(csGetCounter(ft, InputNr) < NrOfChanges) {
		if(aNotHalt || GetAsyncKeyState(VK_ESCAPE) != 0) return;
		if(TermInputNr > 0) if(GetInput(TermInputNr)) return;
		if(GetInputs() == ftiError)
			throw CFishFaceException(30, "InterfaceProblem.WaitForChange");
	}
}

void CFishFace::WaitForHigh(int InputNr) {
	WaitForInput(InputNr, false);
	WaitForInput(InputNr, true);
}

void CFishFace::WaitForInput(int InputNr, bool OnOff) {
	if(!aOpen) throw CFishFaceException(31, "KeinOpen.WaitForInput");
	do {
		if(aNotHalt || GetAsyncKeyState(VK_ESCAPE) != 0) return;
		if(GetInputs() == ftiError)
			throw CFishFaceException(32, "InterfaceProblem.WaitForInput");
		Sleep(aPollInterval);
	} while(!GetInput(InputNr) == OnOff);
}

void CFishFace::WaitForLow(int InputNr) {
	WaitForInput(InputNr, true);
	WaitForInput(InputNr, false);
}

int CFishFace::WaitForMotors(unsigned int mSek, int M1, int M2, int M3, int M4, 
														 int M5, int M6, int M7, int M8) {
	const int mM3[] = {0, 0x3, 0xC, 0x30, 0xC0, 0x300, 0xC00, 0x3000, 0xC000};
	int c;
	unsigned int t;
	if(!aOpen) throw CFishFaceException(33, "KeinOpen.WaitForMotors");
	t = GetTickCount() + mSek;
	do {
		c = 0;
		if(M1) c += csGetMotors(ft) & mM3[M1];
		if(M2) c += csGetMotors(ft) & mM3[M2];
		if(M3) c += csGetMotors(ft) & mM3[M3];
		if(M4) c += csGetMotors(ft) & mM3[M4];
		if(M5) c += csGetMotors(ft) & mM3[M5];
		if(M6) c += csGetMotors(ft) & mM3[M6];
		if(M7) c += csGetMotors(ft) & mM3[M7];
		if(M8) c += csGetMotors(ft) & mM3[M8];

		if(!c) return ftiEnd;

		// --- Abschalten der Motoren der Parameterleiste -----
		if(aNotHalt || GetAsyncKeyState(VK_ESCAPE) != 0) {
			if(M1) csSetMotor(ft, M1, ftiOff);
			if(M2) csSetMotor(ft, M2, ftiOff);
			if(M3) csSetMotor(ft, M3, ftiOff);
			if(M4) csSetMotor(ft, M4, ftiOff);
			if(M5) csSetMotor(ft, M5, ftiOff);
			if(M6) csSetMotor(ft, M6, ftiOff);
			if(M7) csSetMotor(ft, M7, ftiOff);
			if(M8) csSetMotor(ft, M8, ftiOff);
		}
		if(GetInputs() == ftiError)
			throw CFishFaceException(34, "InterfaceProblem.WaitForMotors");
		if(aNotHalt) return ftiNotHalt;
		if(GetAsyncKeyState(VK_ESCAPE) != 0)  return ftiESC;
		Sleep(aPollInterval);
	} while(!mSek || (t > GetTickCount()));
	return ftiTime;
}

int CFishFace::WaitForPositionDown(int InputNr, int Counter, int Position, int TermInputNr) {
	if(!aOpen) throw CFishFaceException(35, "KeinOpen.WaitForPositionDown");
	WaitForChange(InputNr, Counter - Position, TermInputNr);
	return Counter - csGetCounter(ft, InputNr);
}

int CFishFace::WaitForPositionUp(int InputNr, int Counter, int Position, int TermInputNr) {
	if(!aOpen) throw CFishFaceException(36, "KeinOpen.WaitForPositionUp");
	WaitForChange(InputNr, Position - Counter, TermInputNr);
	return csGetCounter(ft, InputNr) + Counter;
}

void CFishFace::WaitForTime(unsigned int mSek) {
	if(!aOpen) throw CFishFaceException(37, "KeinOpen.WaitForTime");
	Pause(mSek);
}
