// -------------------------------------------------------------------------------------
//
// FishFa30.H                                                     Ulrich Mller 13.08.04
//                                                                PCS1700       VC++ 6.0
// H-File zu den Klassen CFishFace ... in FishFa30.CPP
//
// -------------------------------------------------------------------------------------

#ifndef _FishFa30_H_
#define _FishFa30_H_

#include "csFish30.H"

// --------- enums und const ---------------------------------------------------------------

const int ftiFehler = 0x0001FFFF, ftiError = ftiFehler;

enum Dir  {ftiLeft  = 1, ftiRight  = 2, ftiOff = 0, ftiOn  = 1,
					 ftiLinks = 1, ftiRechts = 2, ftiAus = 0, ftiEin = 1};
enum nr   {ftiEX = 0, ftiEY = 1,
					 ftiE1  = 1,  ftiE2  = 2,  ftiE3  = 3,  ftiE4  = 4,
					 ftiE5  = 5,  ftiE6  = 6,  ftiE7  = 7,  ftiE8  = 8,
					 ftiE9  = 9,  ftiE10 = 10, ftiE11 = 11, ftiE12 = 12,
					 ftiE13 = 13, ftiE14 = 14, ftiE15 = 15, ftiE16 = 16,
					 ftiM1  = 1,  ftiM2  = 2,  ftiM3  = 3,  ftiM4  = 4,
					 ftiM5  = 5,  ftiM6  = 6,  ftiM7  = 7,  ftiM8  = 8};
enum Speed{ftiNome = 0, ftiHalf = 7, ftiFull = 15};
enum Wait {ftiEnd = 0,  ftiTime = 1, ftiNotHalt = 2, ftiESC = 3};
enum Port {ftiLPT = 0, ftiCOM1 = 1, ftiCOM2 = 2, ftiCOM3 = 3, ftiCOM4 = 4,
					 ftiLPT1 = 9, ftiLPT2 = 10};

// ----- class CFishFaceException  ----------------------------------------------------------

class CFishFaceException {
private:
	int   aNr;
	char* aText;
public:
	CFishFaceException(int nr, char* text);
	char* Text() const;
	int   Nr() const;
};

inline char* CFishFaceException::Text() const {return aText;}
inline int   CFishFaceException::Nr()   const {return aNr;}

// ------- class CFishFace -------------------------------------------------------------------

class CFishFace {
private:
	int  ft;
	int  aPortNr;
	bool aOpen;
	bool aNotHalt;
	int  aLPTDelay;
	int  aLPTAnalog;
	int  aSlave;
	int  aPollInterval;
	int  aAnalogScan;

public:
	CFishFace();
	CFishFace(bool AnalogScan, bool Slave, int PollInterval);
	CFishFace(bool AnalgoScan, bool Slave, int PollInterval, int LPTAnalog, int LPTDelay);
	~CFishFace();

	bool AnalogScan() const;
	int  AnalogsEX()  const;
	int  AnalogsEY()  const;
	void ClearCounter(int InputNr);
	void ClearCounters();
	void ClearMotors();
	void CloseInterface();
	bool Finish();
	bool Finish(int InputNr);
	int  GetAnalog(int AnalogNr);
	int  GetAnalogDirect(int AnalogNr);
	int  GetCounter(int InputNr) const;
	bool GetInput(int InputNr);
	int  GetInputs();
	int  Inputs() const;
	int  LPTAnalog() const;
	int  LPTDelay()  const;
	bool NotHalt() const;
	void NotHalt(bool value);
	void OpenInterface(int PortNr);
	int  Outputs() const;
	void Outputs(int value);
	void Pause(unsigned int mSek);
	int  PollInterval() const;
	void SetCounter(int InputNr, int value);
	void SetLamp(int LampNr, int OnOff);
	void SetMotor(int MotorNr, int Direction);
	void SetMotor(int MotorNr, int Direction, int Speed);
	void SetMotor(int MotorNr, int Direction, int Speed, int Counter);
	void SetMotors(int MotorStatus);
	void SetMotors(int MotorStatus, int SpeedStatus);
	void SetMotors(int MotorStatus, int SpeedStatus, int ModeStatus);
	bool Slave() const;
	char* Version() const;
	void WaitForChange(int InputNr, int NrOfChanges, int TermInputNr = 0);
	void WaitForHigh(int InputNr);
	void WaitForInput(int InputNr, bool OnOff = true);
	void WaitForLow(int InputNr);
	int  WaitForMotors(unsigned int mSek, int M1, int M2 = 0, int M3 = 0, int M4 = 0,
															int M5 = 0, int M6 = 0, int M7 = 0, int M8 = 0);
	int  WaitForPositionDown(int InputNr, int Counter, int Position, int TermInputNr = 0);
	int  WaitForPositionUp(int InputNr, int Counter, int Position, int TermInputNr = 0);
	void WaitForTime(unsigned int mSek);

};

inline bool CFishFace::AnalogScan() const {return csGetAnalogScan(ft) == 0 ? false : true;}
inline int  CFishFace::AnalogsEX()  const {return csGetAnalog(ft, 0);}
inline int  CFishFace::AnalogsEY()  const {return csGetAnalog(ft, 1);}
inline void CFishFace::ClearCounter(int InputNr) {csSetCounter(ft, InputNr, 0);}
inline void CFishFace::ClearCounters() {csClearCounters(ft);}
inline int  CFishFace::GetCounter(int InputNr) const {return csGetCounter(ft, InputNr);}
inline int  CFishFace::Inputs()     const {return csGetInputs(ft);}
inline int  CFishFace::LPTAnalog()  const {return csGetLPTAnalog(ft);}
inline int  CFishFace::LPTDelay()   const {return csGetLPTDelay(ft);}
inline bool CFishFace::NotHalt()    const {return aNotHalt;}
inline void CFishFace::NotHalt(bool value) {aNotHalt = value;}
inline int  CFishFace::Outputs()    const {csGetMotors(ft);}
inline void CFishFace::Outputs(int value) {csSetMotors(ft, value);}
inline int  CFishFace::PollInterval() const {return aPollInterval;}
inline void CFishFace::SetCounter(int InputNr, int value) {csSetCounter(ft, InputNr, value);}
inline bool CFishFace::Slave() const {return csGetSlave(ft) == 0 ? false : true;}
inline char* CFishFace::Version() const {return "3.1.0.1";}


#endif
